
#ifndef GLUCOSE_SERIAL_GLUCOSE_H
#define GLUCOSE_SERIAL_GLUCOSE_H

#include <cmath>
#include <limits>

#include "model/Parameters.h"
#include "model/Segment.h"
#include "model/FitnessResult.h"

/*
 * Trida s metodami pro vypocet vhodnosti parametru
 */
class Glucose {
public:
    /*
     * Vypocita vhodnost parametru
     *
     * @param parameters odkaz na parametry
     * @param segment odkaz na segment
     * @return vysledek vhodnosti
     */
    static FitnessResult calculationOfFitness(Parameters *parameters, Segment *segment);
    /*
     * Vypocita model pro jeden cas
     *
     * @param time cas
     * @param parameters odkaz na parametry
     * @param segment odkaz na segment
     * @return vhodnost v danem case
     */
    static double calculationOfModel(double time, Parameters *parameters, Segment *segment);
private:
    /*
     * Vypocita hodnotu Phi
     *
     * @param time cas
     * @param parameters odkaz na parametry
     * @param segment odkaz na segment
     * @return hodnota phi
     */
    static double getPhi(double time, Parameters *parameters, Segment *segment);
    /*
     * Vypocita hodnotu Beta
     *
     * @param time cas
     * @param parameters odkaz na parametry
     * @param segment odkaz na segment
     * @return hodnota beta
     */
    static double getBeta(double time, Parameters *parameters, Segment *segment);
    /*
     * Vypocita hodnotu Phi
     *
     * @param time cas
     * @param parameters odkaz na parametry
     * @param segment odkaz na segment
     * @return hodnota gamma
     */
    static double getGamma(Parameters *parameters, Segment *segment, double phi);
    /*
     * Vypocita hodnotu Phi
     *
     * @param time cas
     * @param parameters odkaz na parametry
     * @param segment odkaz na segment
     * @param hodnota phi
     * @return hodnota D
     */
    static double getD(double time, Parameters *parameters, Segment *segment, double phi);
};

#endif //GLUCOSE_SERIAL_GLUCOSE_H
