#include "NelderMeadAlgorithm.h"

#include "./model/Config.h"
#include "./Glucose.h"

using namespace std;

double NelderMeadAlgorithm::checkParameter(double value, int parameterId) {
    switch (parameterId) {
        case Parameters::P:
            if (Config::bounds.getPMin() > value ) {
                return Config::bounds.getPMin();
            } else if (value > Config::bounds.getPMax()) {
                return Config::bounds.getPMax();
            } else {
                return value;
            }
        case Parameters::CG:
            // cg nesmi byt nula = deleno nulou, pokud nula pridavam 0,000001
            if (Config::bounds.getCgMin() > value ) {
                if (Config::bounds.getCgMin() != 0)
                    return Config::bounds.getCgMin();
                else
                    return Config::bounds.getCgMin()+0.000001;
            } else if (value > Config::bounds.getCgMax()) {
                if (Config::bounds.getCgMax() != 0)
                    return Config::bounds.getCgMax();
                else
                    return Config::bounds.getCgMax()-0.000001;
            } else {
                return value;
            }
        case Parameters::C:
            if (Config::bounds.getCMin() > value) {
                return Config::bounds.getCMin();
            } else if (value > Config::bounds.getCMax()) {
                return Config::bounds.getCMax();
            } else {
                return value;
            }
        case Parameters::DT:
            if (Config::bounds.getDtMin() > value) {
                return Config::bounds.getDtMin();
            } else if (value > Config::bounds.getDtMax()) {
                return Config::bounds.getDtMax();
            } else {
                return value;
            }
        case Parameters::H:
            // h nesmi byt nula = deleno nulou, pokud nula pridavam 0,000001
            if (Config::bounds.getHMin() > value ) {
                if (Config::bounds.getHMin() != 0)
                    return Config::bounds.getHMin();
                else
                    return Config::bounds.getHMin()+0.000001;
            } else if (value > Config::bounds.getHMax()) {
                if (Config::bounds.getHMax() != 0)
                    return Config::bounds.getHMax();
                else
                    return Config::bounds.getHMax()-0.000001;
            } else {
                return value;
            }
        case Parameters::K:
            if (Config::bounds.getKMin() > value) {
                return Config::bounds.getKMin();
            } else if (value > Config::bounds.getKMax()) {
                return Config::bounds.getKMax();
            } else {
                return value;
            }
        default:
            return value;
    }
}

// Xg=(X[1]+...+X[n-1]+X[n])/n
void NelderMeadAlgorithm::getXg(Parameters *xg) {
    double sum, *row;

    for (int i = 0; i < NUMBER_OF_PARAMETERS; i++) {
        sum = 0;
        for (int j = 0; j < NUMBER_OF_PARAMETERS + 1; j++) {
            row = _simplex.getParameters(j)->getParametersArray();
            sum += row[i];
        }
        xg->setParameter(i, checkParameter(sum / NUMBER_OF_PARAMETERS, i));
    }
}

// Xr=A*(Xg-X[n-1])+Xg
void NelderMeadAlgorithm::getXr(Parameters *xg, Parameters *xr) {
    double A = 1;

    for (int i = 0; i < NUMBER_OF_PARAMETERS; i++) {
        xr->setParameter(i, checkParameter(
                A * (xg->getParametersArray()[i] - _simplex.getParameters(6)->getParametersArray()[i]) +
                xg->getParametersArray()[i], i));
    }
}

// Xe=B*(Xr-Xg)+Xr
void NelderMeadAlgorithm::getXe(Parameters *xr, Parameters *xg, Parameters *xe) {
    double B = 2;

    for (int i = 0; i < NUMBER_OF_PARAMETERS; i++) {
        xe->setParameter(i, checkParameter(
                B * (xr->getParametersArray()[i] - xg->getParametersArray()[i]) + xr->getParametersArray()[i], i));
    }

}

// Xc=G*(X[n+1]-Xg)+Xg
void NelderMeadAlgorithm::getXc(Parameters *xg, Parameters *xc) {
    double G = 0.5;

    for (int i = 0; i < NUMBER_OF_PARAMETERS; i++) {
        xc->setParameter(i, checkParameter(
                G * (_simplex.getParameters(6)->getParametersArray()[i] - xg->getParametersArray()[i]) +
                xg->getParametersArray()[i], i));
    }

}

// X[k]=H*(X[k]-X[1])+X[1]
// 2<=k<=n=1
void NelderMeadAlgorithm::reduction() {
    double HA = 0.5;

    for (int i = 1; i < NUMBER_OF_PARAMETERS + 1; i++) {
        for (int j = 0; j < NUMBER_OF_PARAMETERS; j++) {
            _simplex.getParameters(i)->setParameter(j, checkParameter(HA * (_simplex.getParameters(
                    i)->getParametersArray()[j] - _simplex.getParameters(0)->getParametersArray()[j]) +
                                                                      _simplex.getParameters(
                                                                              0)->getParametersArray()[j], j));
        }
    }
}


FitnessResult NelderMeadAlgorithm::findParameters(Segment* segment) {

    //generuje simplex, kde alespon jedna sada vraci platnou hodnotu
    _simplex.generateSimplex();
    bool ok = false;
    int numberOfTest = 0;
    while (true) {
        for (int i = 0; i < 7; ++i) {
            FitnessResult fitnessResult = Glucose::calculationOfFitness(_simplex.getParameters(i), segment);

            if (fitnessResult.getFitness() < numeric_limits<double>::max()) {
                ok = true;
                break;
            }
        }
        if (ok) {
            break;
        }
        // po patem neuspesnem pokusu snizuju hranici
        if (numberOfTest == 4) {
            numberOfTest = -1;
            segment->setNumberOfMinData(segment->getNumberOfMinData()-1);
        }
        numberOfTest++;
        _simplex.generateSimplex();
    }

    _segment = segment;

    Parameters *xg = new Parameters();
    Parameters *xr = new Parameters();
    Parameters *xc = new Parameters();
    Parameters *xe = new Parameters();

    FitnessResult  f_xr, f_xc, f_xe;

    // zacatek Nealder Mead algoritmu
    for (int i = 0; i < Config::iterations; i++) {
        _simplex.sort(&Glucose::calculationOfFitness, _segment);

        //sada parametru ma pozadovanou presnost, konec hledani
        if (_simplex.getBestFitnessResult().getFitness() < Config::epsilon) {
            break;
        }

        getXg(xg);
        getXr(xg, xr);

        f_xr = getFitness(xr);

        if (_simplex.getBestFitnessResult().getFitness() < f_xr.getFitness() && f_xr.getFitness() < _simplex.getSecondWorstFitnessResult().getFitness()) {
            _simplex.setRow(*xr, WORST_FITNESS_RESULT);
        } else {
            if (_simplex.getBestFitnessResult().getFitness() < f_xr.getFitness()) {
                getXc(xg, xc);
                f_xc = getFitness(xc);

                if (f_xc.getFitness() < _simplex.getWorstFitnessResult().getFitness()) {
                    _simplex.setRow(*xc, WORST_FITNESS_RESULT);
                } else {
                    reduction();
                }
            } else {
                getXe(xr, xg, xe);
                f_xe = getFitness(xe);
                if (f_xe.getFitness() < f_xr.getFitness()) {
                    _simplex.setRow(*xe, WORST_FITNESS_RESULT);
                } else {
                    _simplex.setRow(*xr, WORST_FITNESS_RESULT);
                }
            }
        }
    }

    delete xg;
    delete xr;
    delete xc;
    delete xe;

    FitnessResult resultFitness = _simplex.getBestFitnessResult();
    resultFitness.setNumberOfData(_segment->getFilteredMeasuredValues().size());

    return resultFitness;
}

FitnessResult NelderMeadAlgorithm::getFitness(Parameters *parameters) {
    return Glucose::calculationOfFitness(parameters, _segment);
}
