
#ifndef GLUCOSE_SERIAL_NELDERMEADALGORITHM_H
#define GLUCOSE_SERIAL_NELDERMEADALGORITHM_H

#include "./model/Simplex.h"
#include "./model/Bounds.h"
#include "./model/Parameters.h"
#include "./model/Segment.h"
#include "./model/FitnessResult.h"

/*
 * Implementace Nelder-Mead algoritmu
 *
 */
class NelderMeadAlgorithm {
public:
    /*
     * Nalezne nejlepsi parametry v segmentu
     *
     * @param segment odkaz na segment
     */
    FitnessResult findParameters(Segment *segment);

private:
    Simplex _simplex;
    Segment* _segment;

    /*
     * Zkontroluje meze parametru
     *
     * @param value hodnota parametru
     * @param parameterId id parametru
     */
    double checkParameter(double value, int parameterId);
    /*
     * Vrati bod xg
     *
     * @param xg kam se maji ulozit data
     */
    void getXg(Parameters *xg);
    /*
     * Vrati bod xr
     *
     * @param xg bod xg
     * @param xr kam se maji ulozit data
     */
    void getXr(Parameters *xg, Parameters *xr);
    /*
     * Vrati bod xe
     *
     * @param xr bod xr
     * @param xg bod xg
     * @param xe kam se maji ulozit data
     */
    void getXe(Parameters *xr, Parameters *xg, Parameters *xe);
    void getXc(Parameters *xg, Parameters *xc);
    /*
     * Vrati vhodnost sady parametru
     *
     * @parameters odkaz na sadu parametru
     * @return vysledek vhodnosti
     */
    FitnessResult getFitness(Parameters *parameters);
    /*
     * Redukce simplexu
     */
    void reduction();
};

#endif //GLUCOSE_SERIAL_NELDERMEADALGORITHM_H
