#include <chrono>
#include <random>

#include "Simplex.h"
#include "Config.h"


void Simplex::generateSimplex() {
    unsigned seed = chrono::system_clock::now().time_since_epoch().count();
    default_random_engine generator(seed);

    //nastaveni rozsahu
    uniform_real_distribution<double> p(Config::bounds.getPMin(), Config::bounds.getPMax());
    uniform_real_distribution<double> cg(Config::bounds.getCgMin(), Config::bounds.getCgMax());
    uniform_real_distribution<double> c(Config::bounds.getCMin(), Config::bounds.getCMax());
    uniform_real_distribution<double> dt(Config::bounds.getDtMin(), Config::bounds.getDtMax());
    uniform_real_distribution<double> h(Config::bounds.getHMin(), Config::bounds.getHMax());
    uniform_real_distribution<double> k(Config::bounds.getKMin(), Config::bounds.getKMax());

    for (int i = 0; i < NUMBER_OF_PARAMETERS + 1; i++) {
        _simplex[i].setP(p(generator));
        _simplex[i].setCg(cg(generator));
        _simplex[i].setC(c(generator));
        _simplex[i].setDt(dt(generator));
        _simplex[i].setH(h(generator));
        _simplex[i].setK(k(generator));
    }
}

void Simplex::sort(FitnessResult (*fitness) (Parameters*, Segment*), Segment *segment) {

    // ziskani vhodnosti pro vsechny sady
    for (int i = 0; i < NUMBER_OF_PARAMETERS + 1; i++) {
        fitnessResults[i] = fitness(&_simplex[i], segment);
    }

    FitnessResult tmp;

    // serazeni vysledku
    for (int i = 0; i < NUMBER_OF_PARAMETERS; i++) {
        for (int j = 0; j < NUMBER_OF_PARAMETERS - i; j++) {
            if (fitnessResults[j+1].getFitness() < fitnessResults[j].getFitness()) {
                tmp = fitnessResults[j+1];
                fitnessResults[j+1] = fitnessResults[j];
                fitnessResults[j] = tmp;
            }
        }
    }

    // prehazeni sad podle vhodnoti od nejlepsi
    for (int i = 0; i < NUMBER_OF_PARAMETERS + 1; ++i) {
        _simplex[i] = fitnessResults[i].getParameters();
    }
}

Parameters* Simplex::getParameters(int row) {
    return &_simplex[row];
}

void Simplex::setRow(Parameters parameters, int row) {
    _simplex[row] = parameters;
}

FitnessResult Simplex::getBestFitnessResult() {
    return fitnessResults[BEST_FITNESS_RESULT];
}

FitnessResult Simplex::getSecondWorstFitnessResult() {
    return fitnessResults[SECOND_WORST_FITNESS_RESULT];
}

FitnessResult Simplex::getWorstFitnessResult() {
    return fitnessResults[WORST_FITNESS_RESULT];
}
