#ifndef GLUCOSE_SERIAL_SIMPLEX_H
#define GLUCOSE_SERIAL_SIMPLEX_H

#include <vector>

#include "Bounds.h"
#include "Parameters.h"
#include "Segment.h"

/*
 * Pocet parametru
 */
#define NUMBER_OF_PARAMETERS 6
/*
 * id nejlepsi sady parametru
 */
#define BEST_FITNESS_RESULT 0
/*
 * id druhe nejhorsi sady parametru
 */
#define WORST_FITNESS_RESULT 5
/*
 * id nejhorsi sady parametru
 */
#define SECOND_WORST_FITNESS_RESULT 6

using namespace std;

/*
 * Trida reprezentujici simplex
 */
class Simplex {
public:
    /*
     * Vygeneruje parametry simplexu podle mezi
     */
    void generateSimplex();
    /*
     * Seradi simplex podle vhodnosti od nejlepsi
     *
     * @param odkaz na funkci podle ktere se ma radit
     * @param segment odkaz na segment
     */
    void sort(FitnessResult (*fitness) (Parameters*, Segment*), Segment* segment);
    /*
     * Vrati radek simplex
     *
     * @param row radek simplexu
     * @return parametry na radce simplexu
     */
    Parameters* getParameters(int row);
    /*
     * Nastavi parametry na radek siplexu
     *
     * @param parameters parametry
     * @param row radek simplexu
     */
    void setRow(Parameters parameters, int row);

    /*
     * Vrati nejlepsi vysledek vhodnosti
     *
     * return nejlepsi vysledek vhodnosti
     */
    FitnessResult getBestFitnessResult();
    /*
     * Vrati druhy nejhorsi vysledek vhodnosti
     *
     * return druhy nejhorsi vysledek vhodnosti
     */
    FitnessResult getSecondWorstFitnessResult();
    /*
    * Vrati nejhorsi vysledek vhodnosti
    *
    * return nejhorsi vysledek vhodnosti
    */
    FitnessResult getWorstFitnessResult();

private:
    Parameters _simplex[NUMBER_OF_PARAMETERS + 1];
    FitnessResult fitnessResults[NUMBER_OF_PARAMETERS + 1];
};

#endif //GLUCOSE_SERIAL_SIMPLEX_H
