
#include "DataLoader.h"

#include <fstream>

#include "./../../include/sqlite/sqlite3.h"

vector<Segment*> DataLoader::loadDataFromDatabase(string path) {

    sqlite3 *db;
    string sql;
    sqlite3_stmt *statement;

    if (!ifstream(path) || sqlite3_open(path.c_str(), &db) != SQLITE_OK) {
        throw "Can't open database: " + path;
    }

    int currentSegmentId = -1;

    vector<Segment*> segments;
    vector<MeasuredValue*> measuredValues;

    // sql dotaz s prevodem casu
    sql = "SELECT "
               "id, "
               "(strftime('%s', datetime(measuredat)) - strftime('%s','1970-01-01') + 2209161600) * (cast ( ( 1 ) as real ) / (24.0*60.0*60.0)) AS date, "
               "blood, "
               "ist, "
               "segmentid "
           "FROM "
               "measuredvalue "
           "ORDER BY "
               "segmentid, "
               "measuredat";

    if (sqlite3_prepare_v2(db, sql.c_str(), -1, &statement, 0) == SQLITE_OK) {
        int result = 0;
        while (true) {
            result = sqlite3_step(statement);
            if (result == SQLITE_ROW) {

                MeasuredValue *measuredValue = new MeasuredValue();

                measuredValue->setMeasuredat(sqlite3_column_double(statement, 1));
                measuredValue->setBlood(sqlite3_column_double(statement, 2));
                measuredValue->setIst(sqlite3_column_double(statement, 3));
                int segmentId =  sqlite3_column_int(statement, 4);

                // prvni segment, je potreba nstavit aktualni id segmentu
                if (currentSegmentId == -1) {
                    currentSegmentId = segmentId;
                }

                // id segmentu se nerovna s aktualnim = dalsi segment
                if (currentSegmentId != segmentId) {
                    // ulozeni segmentu
                    Segment *segment = new Segment();
                    segment->setId(currentSegmentId);
                    segment->setMeasuredValues(measuredValues);
                    segments.push_back(segment);
                    currentSegmentId = segmentId;
                    measuredValues.clear();
                }
                measuredValues.push_back(measuredValue);
            } else {
                // ulozeni segmentu
                Segment *segment = new Segment();
                segment->setId(currentSegmentId);
                segment->setMeasuredValues(measuredValues);
                segments.push_back(segment);
                break;
            }
        }
        sqlite3_finalize(statement);
    } else {
        throw "SQL error: " + sql;
    }
    sqlite3_close(db);

    return segments;
}


Bounds DataLoader::loadBounds(string path) {
    Bounds bounds;

    string lines = "";

    string line;
    ifstream file(path);
    if (file.is_open())
    {
        while (getline(file, line)) {
            lines += line + ";";
        }

        bounds.setPMin(getParameterFromString(lines, "pmin="));
        bounds.setPMax(getParameterFromString(lines, "pmax="));

        bounds.setCgMin(getParameterFromString(lines, "cgmin="));
        bounds.setCgMax(getParameterFromString(lines, "cgmax="));

        bounds.setCMin(getParameterFromString(lines, "cmin="));
        bounds.setCMax(getParameterFromString(lines, "cmax="));

        bounds.setDtMin(getParameterFromString(lines, "dtmin="));
        bounds.setDtMax(getParameterFromString(lines, "dtmax="));

        bounds.setHMin(getParameterFromString(lines, "hmin="));
        bounds.setHMax(getParameterFromString(lines, "hmax="));

        bounds.setKMin(getParameterFromString(lines, "kmin="));
        bounds.setKMax(getParameterFromString(lines, "kmax="));

        file.close();
    } else {
        throw "Can't open bounds file: " + path;
    }

    return bounds;
}

double DataLoader::getParameterFromString(string lines, string name) {

    int start = lines.find(name) + string(name).length();
    int end = lines.find(";", start);
    return stod(lines.substr(start, end - start).c_str());

}