//
// Created by 1Max11 on 01.01.2016.
//

#include "MPIService.h"

#include "../model/Config.h"
#include "./DataLoader.h"
#include "./Print.h"
#include "./Message.h"

void MPIService::master(int argc, char **argv, int num_procs, chrono::high_resolution_clock::time_point begin_ts1) {
    MPI_Status status;
    vector<Segment*> segments;

    chrono::high_resolution_clock::time_point begin_ts2;

    int ts1, ts2, tp;

    int state = 1;

    if (argc < 3) {
        cout << "./glusoce_parallel <path of bounds> <path of sqlite> [<parameters>]" << endl;
        cout << "Parameters:" << endl;
        cout << "-i<int>    - number of iterations" << endl;
        cout << "-t<int>    - number of threads" << endl;
        cout << "-e<double> - epsilon" << endl;
        cout << "-m<double> - coefficien of min data" << endl;
        state = 0;
    } else {
        Config::setProgram(argc, argv);
    }

    if (state == 1) {
        try {
            Config::bounds = DataLoader::loadBounds(argv[1]);
            segments = DataLoader::loadDataFromDatabase(argv[2]);
        } catch (string& e) {
            cout << e << endl;
            state = 0;
        }
    }

    // odesle vsem procesum stav, 1 = v poradku, 0 = konec programu
    for (int i = 1; i < num_procs; ++i) {
        MPI_Send(&state, 1, MPI_INT, i, 5, MPI_COMM_WORLD); // odesilam vsem stav
    }

    // soubory nenacteny = konec programu
    if (state == 0) {
        MPI_Finalize();
        exit(0);
    }


    Print::printHeader(&Config::bounds, argv[1], argv[2], num_procs);

    ProcessStatistic *processStatistics[num_procs];

    if (num_procs > 1) {
        int processWork[num_procs];
        for (int i = 0; i < num_procs; ++i) {
            processWork[i] = 0;
        }

        map<int, Segment *> map;
        for (unsigned int i = 0; i < segments.size(); i++) {
            map[segments[i]->getId()] = segments[i];
        }

        //odesle vsem procesum nastaveni
        for (int i = 1; i < num_procs; ++i) {
            sendMessage(Message::createBoundsMessage(&Config::bounds), i);
            sendMessage(Message::createConfigMessage(), i);
        }

        unsigned int i = 0;
        Segment *sendSegment;
        int segmendId, idProces;
        int barriera = num_procs - 1;
        int numberOfProcessed = 1;

        auto end_ts1 = chrono::high_resolution_clock::now();
        ts1 =  chrono::duration_cast<std::chrono::milliseconds>(end_ts1-begin_ts1).count();

        //ZACATEK PARALELNI CASTI
        auto begin_tp = chrono::high_resolution_clock::now();
        while (true) {
            // proces chce data
            MPI_Recv(&idProces, 1, MPI_INT, MPI_ANY_SOURCE, 2, MPI_COMM_WORLD, &status);
            if (i < segments.size()) {
                cout << '\r' << numberOfProcessed++ << "/" << segments.size() << " completed";
                cout.flush();
                sendSegment = segments[i];
                segmendId = segments[i]->getId();
                i++;
            } else {
                segmendId = -1;
                barriera--;
            }
            // procesu odesilam id segmentu, -1 = uz neni zadny segment
            MPI_Send(&segmendId, 1, MPI_INT, idProces, 3, MPI_COMM_WORLD);
            if (segmendId != -1) {
                processWork[idProces]++; // prictu procesu dalsi zpracovany segment
                // odeslu segment procesu
                sendMessage(Message::createMeasuredValueMessage(sendSegment->getMeasuredValues()), idProces);
            } else {
                // prijimam vsechny vysledky procesu
                for (int i = 0; i < processWork[idProces]; ++i) {
                    int recvIdSegment;
                    MPI_Recv(&recvIdSegment, 1, MPI_INT, idProces, 2, MPI_COMM_WORLD, &status); //prijmuni id segmentu
                    map[recvIdSegment]->setResult(*Message::parseFitnessResultMessage(recvMessage(idProces, &status)));
                }

                int processRunTime;
                MPI_Recv(&processRunTime, 1, MPI_INT, idProces, 2, MPI_COMM_WORLD, &status);

                ProcessStatistic *stat = new ProcessStatistic();
                stat->setNumberOfProcessedSegments(processWork[idProces]);
                stat->setRunTime(processRunTime);
                stat->setProcessId(idProces);

                processStatistics[idProces] = stat;
            }
            if (barriera == 0) {
                cout << endl << endl;
                break;
            }
        }
        //KONEC PARALELNI CASTI
        auto end_tp = chrono::high_resolution_clock::now();
        tp =  chrono::duration_cast<std::chrono::milliseconds>(end_tp-begin_tp).count();


        begin_ts2 = chrono::high_resolution_clock::now();

        ProcessStatistic *stat = new ProcessStatistic();
        stat->setNumberOfProcessedSegments(-1);
        stat->setRunTime(tp);
        stat->setProcessId(0);

        processStatistics[0] = stat;

    } else {

        auto end_ts1 = chrono::high_resolution_clock::now();
        ts1 =  chrono::duration_cast<std::chrono::milliseconds>(end_ts1-begin_ts1).count();
        auto begin_tp = chrono::high_resolution_clock::now();
        // proces sam zpracuje vsechny segmenty
        NelderMeadAlgorithm nma;
        for (unsigned  int i = 0; i < segments.size(); i++) {
            segments[i]->setNumberOfMinData(segments[i]->getFilteredMeasuredValues().size() * Config::minDataCoefficient);
            segments[i]->setResult(nma.findParameters(segments[i]));
            cout << '\r' << i + 1 << "/" << segments.size() << " completed";
            cout.flush();
        }
        cout << endl << endl;
        auto end_tp = chrono::high_resolution_clock::now();
        tp =  chrono::duration_cast<std::chrono::milliseconds>(end_tp-begin_tp).count();

        begin_ts2 = chrono::high_resolution_clock::now();
        ProcessStatistic *stat = new ProcessStatistic();
        stat->setNumberOfProcessedSegments(segments.size());
        stat->setRunTime(tp);
        stat->setProcessId(0);

        processStatistics[0] = stat;
    }

    Print::printResults(segments);

    try {
        Storage::saveResultToDatabase(argv[2], segments);
    } catch (string& e) {
        cout << e << endl;
    }

    auto end_ts2 = chrono::high_resolution_clock::now();
    ts2 =  chrono::duration_cast<std::chrono::milliseconds>(end_ts2-begin_ts2).count();

    Print::printStatistics(processStatistics, num_procs, ts1 + ts2, tp);

    for (unsigned i = 0; i < segments.size(); i++) {
        delete segments[i];
    }
}

void MPIService::slave(int my_id) {
    auto begin = chrono::high_resolution_clock::now();

    MPI_Status status;

    int state;

    MPI_Recv(&state, 1, MPI_INT, ROOT_PROCESS, 5, MPI_COMM_WORLD, &status); //ziskam stav

    if (state == 0) {
        MPI_Finalize();
        exit(0);
    }

    vector<Segment*> segments;

    Config::bounds = *Message::parseBoundsMessage(recvMessage(ROOT_PROCESS, &status));
    Message::setConfigFromMessage(recvMessage(ROOT_PROCESS, &status));

    NelderMeadAlgorithm *nma = new NelderMeadAlgorithm();

    while (true) {
        MPI_Send(&my_id, 1, MPI_INT, ROOT_PROCESS, 2, MPI_COMM_WORLD); // posilam ze chci data


        int segmentId;
        MPI_Recv(&segmentId, 1, MPI_INT, ROOT_PROCESS, 3, MPI_COMM_WORLD, &status); // id segmentu, ktery mam zpracovat

        if (segmentId != -1) {
            Segment *seg = new Segment();

            seg->setId(segmentId);

            seg->setMeasuredValues(Message::parseMeasuredValueMessage(recvMessage(ROOT_PROCESS, &status))); // prijem namerenich dat
            seg->setNumberOfMinData(Config::minDataCoefficient * seg->getFilteredMeasuredValues().size());

            seg->setResult(nma->findParameters(seg));

            segments.push_back(seg);

        } else {
            //odesilam vsechny vysledky
            for (unsigned int i = 0; i < segments.size(); ++i) {
                segmentId = segments[i]->getId();

                MPI_Send(&segmentId, 1, MPI_INT, ROOT_PROCESS, 2, MPI_COMM_WORLD); // odsesilam id segmentu

                FitnessResult a = segments[i]->getResult();

                sendMessage(Message::createFitnessMessage(&a), ROOT_PROCESS);
            }

            auto end = chrono::high_resolution_clock::now();

            // odesilam dobu behu
            int processRunTime = chrono::duration_cast<std::chrono::milliseconds>(end-begin).count();;
            MPI_Send(&processRunTime, 1, MPI_INT, ROOT_PROCESS, 2, MPI_COMM_WORLD);
            break;
        }
    }
    for (unsigned i = 0; i < segments.size(); i++) {
        delete segments[i];
    }
    delete nma;
}

void MPIService::sendMessage(string message, int id) {
    int len = message.size();
    char sendMessage[len];
    strcpy(sendMessage, message.c_str());
    MPI_Send(&len, 1, MPI_INT, id, 1, MPI_COMM_WORLD);
    MPI_Send(sendMessage, len, MPI_CHAR, id, 0, MPI_COMM_WORLD);
}

string MPIService::recvMessage(int id, MPI_Status *status) {
    //MPI_Status status;
    int len;
    MPI_Recv(&len, 1, MPI_INT, id, 1, MPI_COMM_WORLD, status);
    char recvMessage[len];
    MPI_Recv(recvMessage, len, MPI_CHAR, id, 0, MPI_COMM_WORLD, status);
    return string(recvMessage);
}


