//
// Created by 1Max11 on 27.12.2015.
//

#include <sstream>
#include "../model/Config.h"
#include "Message.h"

string Message::createBoundsMessage(Bounds *bounds) {

    stringstream ss;

    ss << bounds->getPMax() << '\n';
    ss << bounds->getPMin() << '\n';

    ss << bounds->getCgMax() << '\n';
    ss << bounds->getCgMin() << '\n';

    ss << bounds->getCMax() << '\n';
    ss << bounds->getCMin() << '\n';

    ss << bounds->getDtMax() << '\n';
    ss << bounds->getDtMin() << '\n';

    ss << bounds->getHMax() << '\n';
    ss << bounds->getHMin() << '\n';

    ss << bounds->getKMax() << '\n';
    ss << bounds->getKMin();

    return ss.str();
}

Bounds* Message::parseBoundsMessage(string message) {

    Bounds *bounds = new Bounds();

    istringstream iss(message);
    string line;
    getline(iss, line);
    bounds->setPMax(atof(line.c_str()));
    getline(iss, line);
    bounds->setPMin(atof(line.c_str()));
    getline(iss, line);
    bounds->setCgMax(atof(line.c_str()));
    getline(iss, line);
    bounds->setCgMin(atof(line.c_str()));
    getline(iss, line);
    bounds->setCMax(atof(line.c_str()));
    getline(iss, line);
    bounds->setCMin(atof(line.c_str()));
    getline(iss, line);
    bounds->setDtMax(atof(line.c_str()));
    getline(iss, line);
    bounds->setDtMin(atof(line.c_str()));
    getline(iss, line);
    bounds->setHMax(atof(line.c_str()));
    getline(iss, line);
    bounds->setHMin(atof(line.c_str()));
    getline(iss, line);
    bounds->setKMax(atof(line.c_str()));
    getline(iss, line);
    bounds->setKMin(atof(line.c_str()));

    return bounds;
}

string Message::createConfigMessage() {

    stringstream ss;

    ss << Config::iterations << '\n';
    ss << Config::epsilon << '\n';
    ss << Config::minDataCoefficient + '\0';

    return ss.str();
}

void Message::setConfigFromMessage(string message) {
    istringstream iss(message);
    string line;
    getline(iss, line);
    Config::iterations = atoi(line.c_str());
    getline(iss, line);
    Config::epsilon = atof(line.c_str());
    getline(iss, line);
    Config::minDataCoefficient = atof(line.c_str());
}

string Message::createMeasuredValueMessage(vector<MeasuredValue *> measuredValue) {

    stringstream ss;
    ss.precision(100);

    for (auto mv : measuredValue) {
        ss << mv->getMeasuredat() << '\n';
        ss << mv->getBlood() << '\n';
        ss << mv->getIst() << '\n';
    }
    string message = ss.str();
    message = message.substr(0, message.size()-1) + '\0'; //odstrani odradkovani nakonci
    return message;
}

vector<MeasuredValue *> Message::parseMeasuredValueMessage(string message) {
    vector<MeasuredValue*> measuredValues;

    istringstream iss(message);
    string line;
    while (getline(iss, line)) {
        MeasuredValue *mv = new MeasuredValue();

        mv->setMeasuredat(atof(line.c_str()));
        getline(iss, line);
        mv->setBlood(atof(line.c_str()));
        getline(iss, line);
        mv->setIst(atof(line.c_str()));

        measuredValues.push_back(mv);
    }
    return measuredValues;
}

string Message::createFitnessMessage(FitnessResult *fitnessResult) {
    stringstream ss;
    ss.precision(20);

    ss << fitnessResult->getParameters().getP() << '\n';
    ss << fitnessResult->getParameters().getCg() << '\n';
    ss << fitnessResult->getParameters().getC() << '\n';
    ss << fitnessResult->getParameters().getDt() << '\n';
    ss << fitnessResult->getParameters().getH() << '\n';
    ss << fitnessResult->getParameters().getK() << '\n';
    ss << fitnessResult->getNumberOfProcessedData() << '\n';
    ss << fitnessResult->getNumberOfData() << '\n';
    ss << fitnessResult->getFitness() << '\0';

    return ss.str();
}

FitnessResult* Message::parseFitnessResultMessage(string message) {
    istringstream iss(message);

    Parameters parameters;

    string line;
    getline(iss, line);
    parameters.setP(atof(line.c_str()));
    getline(iss, line);
    parameters.setCg(atof(line.c_str()));
    getline(iss, line);
    parameters.setC(atof(line.c_str()));
    getline(iss, line);
    parameters.setDt(atof(line.c_str()));
    getline(iss, line);
    parameters.setH(atof(line.c_str()));
    getline(iss, line);
    parameters.setK(atof(line.c_str()));

    FitnessResult *fitnessResult = new FitnessResult();
    fitnessResult->setParameters(parameters);

    getline(iss, line);
    fitnessResult->setNumberOfProcessedData(atoi(line.c_str()));
    getline(iss, line);
    fitnessResult->setNumberOfData(atoi(line.c_str()));
    getline(iss, line);
    fitnessResult->setFitness(atof(line.c_str()));

    return fitnessResult;
}
