
#include <iostream>
#include <iomanip>

#include <chrono>
#include "src/utils/Print.h"

#include "src/manager/ThreadsManager.h"

using namespace std;

/*
 * Hlavni funkce aplikace
 *
 * @param argc pocet parametru
 * @param argv 1       - cesta k mezim (povinne)
 *             2       - cesta k sqlite databazi (povinne)
 *             3 a vic - parametry programu
 *                  -i<int>    - pocet interaci
 *                  -t<int>    - pocet vlaken
 *                  -e<double> - hodnota pod kterou budeme brat vysledek
 *                  -m<double> - koeficien min dat
 */
int main(int argc, char **argv) {
    int ts1, ts2, tp;
    auto begin_ts1 = chrono::high_resolution_clock::now();

    if (argc < 3) {
        cout << "./glusoce_parallel <path of bounds> <path of sqlite> [<parameters>]" << endl;
        cout << "Parameters:" << endl;
        cout << "-i<int>    - number of iterations" << endl;
        cout << "-t<int>    - number of threads" << endl;
        cout << "-e<double> - epsilon" << endl;
        cout << "-m<double> - coefficien of min data" << endl;
        exit(0);
    }

    // nastaveni programu
    Config::setProgram(argc, argv);

    vector<Segment *> segments;
    try {
        // nastaveni mezi
        Config::bounds = DataLoader::loadBounds(argv[1]);
        // nacteni segmentu
        segments = DataLoader::loadDataFromDatabase(argv[2]);
    } catch (string& e) {
        cout << e << endl;
        exit(0);
    }

    Print::printHeader(&Config::bounds, argv[1], argv[2]);

    ThreadsManager threadsManager;

    auto end_ts1 = chrono::high_resolution_clock::now();
    ts1 = chrono::duration_cast<std::chrono::milliseconds>(end_ts1-begin_ts1).count();


    // ZACATEK PARALELNI CASTI
    auto begin_tp = chrono::high_resolution_clock::now();
    // spusteni vlaken, ktere hledaji parametry
    threadsManager.run(segments);
    auto end_tp = chrono::high_resolution_clock::now();
    // KONEC PARALELNI CASTI
    tp =  chrono::duration_cast<std::chrono::milliseconds>(end_tp-begin_tp).count();


    auto begin_ts2 = chrono::high_resolution_clock::now();

    try {
        // ulozeni vysledku do databaze
        Storage::saveResultToDatabase(argv[2], segments);
    } catch (string& e) {
        cout << e << endl;
        exit(0);
    }

    Print::printResults(segments);

    auto end_ts2 = chrono::high_resolution_clock::now();
    ts2 = std::chrono::duration_cast<std::chrono::milliseconds>(end_ts2-begin_ts2).count();

    Print::printStatistics(threadsManager.getStatistics(), Config::numberOfThreads, ts1 + ts2, tp);

    // uvolneni pameti
    for (auto p : segments)
        delete p;

    return 0;
}