
#include "Glucose.h"

FitnessResult Glucose::calculationOfFitness(Parameters *parameters, Segment *segment) {

    FitnessResult fitnessResult;
    fitnessResult.setParameters(*parameters);

    double sumOfResult = 0, sumOfResultPower = 0, sumOfRelativeMistake = 0;
    int count = 0;

    for (MeasuredValue measuredValue : segment->getFilteredMeasuredValues()) {

        double fitness = calculationOfModel(measuredValue.getMeasuredat(), parameters, segment);

        // pro dany cas nelze spocitat, cas preskakuju
        if (fitness == numeric_limits<double>::max()) {
            continue;
        }

        sumOfResult += fitness;
        sumOfResultPower += pow(fitness, 2);
        sumOfRelativeMistake += abs(measuredValue.getBlood() - fitness) / measuredValue.getBlood();
        count++;
    }

    fitnessResult.setNumberOfProcessedData(count);

    if (count == 0 || count < segment->getNumberOfMinData()) {
        fitnessResult.setFitness(numeric_limits<double>::max());
    }else  if (count == 1) {
        fitnessResult.setFitness(sumOfRelativeMistake);
    } else {
        fitnessResult.setFitness(sumOfRelativeMistake + sqrt((1.0 / count * sumOfResultPower) - (pow(sumOfResult / count , 2))));
    }
    return fitnessResult;
}

double Glucose::calculationOfModel(double time, Parameters *parameters, Segment *segment) {

    double D, beta, alpha, phi;

    double startTime = segment->getFilteredMeasuredValues()[0].getMeasuredat();
    double endTime = segment->getFilteredMeasuredValues()[segment->getFilteredMeasuredValues().size() - 1].getMeasuredat();


    phi = getPhi(time, parameters, segment);

    //phi musi byt v intervalu mereni
    if (startTime <= phi && phi <= endTime) {

        D = getD(time, parameters, segment, phi);

        // odmocnina z D
        if (D > 0) {
            beta = getBeta(time, parameters, segment);
            alpha = parameters->getCg();

            // jinak deleno nulou
            if (alpha != 0) {
                return (beta * -1 + sqrt(D)) / (2 * alpha);
            }
        }
    }
    return numeric_limits<double>::max();
}

double Glucose::getPhi(double time, Parameters *parameters, Segment *segment) {

    double startTime = segment->getFilteredMeasuredValues()[0].getMeasuredat();
    double endTime = segment->getFilteredMeasuredValues()[segment->getFilteredMeasuredValues().size() - 1].getMeasuredat();

    // time - h musi byt mezi intervalem mereni, h nesmi byt nula = deleno nulou
    if (startTime <= (time - parameters->getH()) && (time - parameters->getH()) <= endTime && parameters->getH() != 0) {
        return time + parameters->getDt() + parameters->getK() * ((segment->getInterpolationIst(time) - segment->getInterpolationIst(time - parameters->getH())) / parameters->getH());
    } else {
        return numeric_limits<double>::max();
    }
}

double Glucose::getBeta(double time, Parameters *parameters, Segment *segment) {

    return parameters->getP() - parameters->getCg() * segment->getInterpolationIst(time);

}

double Glucose::getGamma(Parameters *parameters, Segment *segment, double phi) {

    return parameters->getC() - segment->getInterpolationIst(phi);

}

double Glucose::getD(double time, Parameters *parameters, Segment *segment, double phi) {

    double beta = getBeta(time, parameters, segment);
    double gamma = getGamma(parameters, segment, phi);

    double alpha = parameters->getCg();

    return beta * beta - 4 * alpha * gamma;

}
