
#include "ThreadsManager.h"

#include <chrono>

#include "../model/Config.h"
#include "../NelderMeadAlgorithm.h"

unsigned int ThreadsManager::segmentId = 0;

vector<Segment*> ThreadsManager::_segments;
pthread_mutex_t ThreadsManager::mutex;

ThreadStatistic** ThreadsManager::_statistics;

ThreadsManager::~ThreadsManager() {
    for (int i = 0; i < Config::numberOfThreads; ++i) {
        delete _statistics[i];
    }
    delete[] _statistics;
    delete[] threads;
}

void ThreadsManager::run(vector<Segment*> segments) {
    _segments = segments;

    _statistics = new ThreadStatistic*[Config::numberOfThreads];

    threads = new pthread_t[Config::numberOfThreads];

    for (int i = 0; i < Config::numberOfThreads; ++i) {
        pthread_create(&threads[i], NULL, ThreadsManager::runNelderMeadAlgorithm, (void*)(intptr_t)i);
    }

    for (int i = 0; i < Config::numberOfThreads; ++i) {
        pthread_join(threads[i], NULL);
    }

    pthread_mutex_destroy(&mutex);
    segmentId = 0;
    cout << endl << endl;
}

Segment* ThreadsManager::getNextSegment() {

    pthread_mutex_lock(&ThreadsManager::mutex);

    Segment *segment;
    if (segmentId < _segments.size()) {
        cout << '\r' << segmentId + 1 << "/" << _segments.size() << " completed";
        cout.flush();
        segment = _segments[segmentId];
        segmentId++;

    } else {
        segment = NULL;
    }

    pthread_mutex_unlock(&ThreadsManager::mutex);

    return segment;
}

void *ThreadsManager::runNelderMeadAlgorithm(void *pThreadId) {

    long threadId = (intptr_t)pThreadId;

    auto begin = chrono::high_resolution_clock::now();

    ThreadStatistic* statistic = new ThreadStatistic();

    int numberOfProcessedSegments = 0;

    NelderMeadAlgorithm nma;
    Segment* segment;

    while ((segment = getNextSegment()) != NULL) {

        numberOfProcessedSegments++;
        segment->setNumberOfMinData(segment->getFilteredMeasuredValues().size() * Config::minDataCoefficient);
        segment->setResult(nma.findParameters(segment));
    }

    statistic->setNumberOfProcessedSegments(numberOfProcessedSegments);

    auto end = chrono::high_resolution_clock::now();

    statistic->setThreadId(threadId);
    statistic->setRunTime(std::chrono::duration_cast<std::chrono::milliseconds>(end-begin).count());

    _statistics[threadId] = statistic;


    return 0;
}

ThreadStatistic **ThreadsManager::getStatistics() {
    return ThreadsManager::_statistics;
}



