
#ifndef GLUCOSE_SERIAL_SEGMENT_H
#define GLUCOSE_SERIAL_SEGMENT_H


#include <vector>

#include "./../../include/alglib/interpolation.h"
#include "MeasuredValue.h"
#include "Parameters.h"
#include "Bounds.h"
#include "FitnessResult.h"

using namespace std;
using namespace alglib;

/*
 * Trida pro uchovavani dat o segmentu
 */
class Segment {
public:
    /*
     * Vrati id segmentu
     *
     * @return id segmentu
     */
    int getId();
    /*
     *  Nastavi id segmentu
     *
     *  @param id id segmentu
     */
    void setId(int id);

    /*
     * Vrati vsechna namerena data v segmentu
     *
     * @return vsechna namerena data
     */
    vector<MeasuredValue> getMeasuredValues();
    /*
     * Nstavi vsechna namerena data v segmentu
     *
     * @param segments segmenty
     */
    void setMeasuredValues(vector<MeasuredValue> segments);

    /*
     * Vrati namerena data bez dat, kde krev nebyla namerena
     *
     * @return vyfiltrovana namerena data
     */
    vector<MeasuredValue> getFilteredMeasuredValues();

    /*
     * Vrati vysledek segmentu
     *
     * @return vysledek segmentu
     */
    FitnessResult getResult();
    /*
     * Nastavi vysledek segmentu
     *
     * @param fitnessResult vysledek segmentu
     */
    void setResult(FitnessResult fitnessResult);

    /*
     * Vrati minimalni pocet dat pro urceni metriky
     *
     * @return minimalni pocet dat
     */
    int getNumberOfMinData();
    /*
     * Nastavi minimalni pocet dat pro urceni metriky
     *
     * @param minimalni pocet dat
     */
    void setNumberOfMinData(int numberOfMinData);

    /*
     * Vrati interpolovane ist
     *
     * @param measuredat cas mereni
     * @return interpolovane ist
     */
    double getInterpolationIst(double measuredat);

private:
    int _id;
    int _numberOfMinData;

    vector<MeasuredValue> _measuredValues;
    vector<MeasuredValue> _filteredMeasuredValues;
    FitnessResult  _result;

    //alglib
    real_1d_array AMeasuredat, AIst;
    spline1dinterpolant spline;

};


#endif //GLUCOSE_SERIAL_SEGMENT_H
