//
// Created by 1Max11 on 29.12.2015.
//

#include "Print.h"


void Print::printHeader(Bounds* bounds, string boundsPath, string databasePath) {
    printf("*********************************************************\n");
    printf("** Info *************************************************\n");
    printf("*********************************************************\n");
    printf("Author:    Martin Blaha\n");
    printf("Algorithm: Nelder-Mead\n");
    printf("Version:   POSIX Threads \n\n");
    printf("Bounds path:   %s\n", boundsPath.c_str());
    printf("Database path: %s\n", databasePath.c_str());
    printf("Config:\n");
    printf("   number of threads    = %d\n", Config::numberOfThreads);
    printf("   iterations           = %d\n", Config::iterations);
    printf("   epsilon              = %g\n", Config::epsilon);
    printf("   min data coefficient = %g\n", Config::minDataCoefficient);
    printf("Bounds:\n");
    printf("   pmin  = %g\n", bounds->getPMin());
    printf("   pmax  = %g\n", bounds->getPMax());
    printf("   cgmin = %g\n", bounds->getCgMin());
    printf("   cgmax = %g\n", bounds->getCgMax());
    printf("   cgmin = %g\n", bounds->getCMin());
    printf("   cmax  = %g\n", bounds->getCMax());
    printf("   dtmin = %g\n", bounds->getDtMin());
    printf("   dtmax = %g\n", bounds->getDtMax());
    printf("   hmin  = %g\n", bounds->getHMin());
    printf("   hmax  = %g\n", bounds->getHMax());
    printf("   kmin  = %g\n", bounds->getKMin());
    printf("   kmax  = %g\n\n\n", bounds->getKMax());

}

void Print::printResults(vector<Segment*> segments) {
    printf("*********************************************************\n");
    printf("** Results **********************************************\n");
    printf("*********************************************************\n");
    printf("Segment ID");
    printf("%11s", "p");
    printf("%15s", "cg");
    printf("%15s", "c");
    printf("%15s", "dt");
    printf("%15s", "h");
    printf("%15s", "k");
    printf("%15s", "fitness");
    printf("%45s\n", "number of processed data/number of data");

    string separator1 = "";
    for (int i = 0; i < 156; i++) {
        separator1 += "=";
    }
    printf("%s\n", separator1.c_str());

    string separator2 = "";
    for (int i = 0; i < 156; i++) {
        separator2 += "-";
    }

    for (unsigned int i = 0; i < segments.size(); ++i) {
        printf("%d", segments[i]->getId());
        printf("%*g", (int)(21 - to_string(segments[i]->getId()).length()), segments[i]->getResult().getParameters().getP());
        printf("%15g", segments[i]->getResult().getParameters().getCg());
        printf("%15g", segments[i]->getResult().getParameters().getC());
        printf("%15g", segments[i]->getResult().getParameters().getDt());
        printf("%15g", segments[i]->getResult().getParameters().getH());
        printf("%15g", segments[i]->getResult().getParameters().getK());
        printf("%15g", segments[i]->getResult().getFitness());
        printf("%42d/%d\n", segments[i]->getResult().getNumberOfProcessedData(),segments[i]->getResult().getNumberOfData());
        printf("%s\n", separator2.c_str());
    }
    printf("\n\n");

}

void Print::printStatistics(ThreadStatistic**threadStatistic, int numberOfTreads, int ts, int tp) {

    printf("*********************************************************\n");
    printf("** Statistics *******************************************\n");
    printf("*********************************************************\n");
    printf("Run time     : %d ms\n", ts + tp);
    printf("Serial time  : %d ms\n", ts);
    printf("Parallel time: %d ms\n\n", tp);
    printf("Thread");
    printf("%35s", "Number of processed segments");
    printf("%14s", "Run time\n");
    printf("=========================================================\n");
    for (int i = 0; i < numberOfTreads; ++i) {
        printf("Thread_%d", threadStatistic[i]->getThreadId());
        printf("%*d", (int)(34 + string("Thread").length() - ("Thread_" + to_string(threadStatistic[i]->getThreadId())).length()),
               threadStatistic[i]->getNumberOfProcessedSegments());
        printf("%11d ms\n", threadStatistic[i]->getRunTime());
        printf("---------------------------------------------------------\n");
    }

}
